Package: re2r
Type: Package
Title: RE2 Regular Expression
Version: 0.2.0
Authors@R: c(
    person("Qin Wenfeng", email = "mail@qinwenfeng.com", role = c("aut", "cre")),
    person("Toby Dylan Hocking", role = "ctb", comment = "benchmarks"),
    person("Marek Gagolewski", role = "ctb", comment = "stringi benchmarks and test cases"),
    person("RE2 developers", role = "ctb", comment = "RE2 library"),
    person("Google Inc.", role = c("ctb", "cph"), comment = "RE2 library"),
    person("Lucent Technologies", role = c("ctb", "cph"), comment = "RE2 library"),
    person("Rob Pike", role = "ctb", comment = "RE2 library"),
    person("Ken Thompson", role = "ctb", comment = "RE2 library"),
    person("Julian Seward", role = c("ctb", "cph"), comment = "valgrind.h"),
    person("Andrzej Krzemienski", role = "ctb", comment = "optional.hpp"),
    person("Jeffrey Avallone", role = "ctb", comment = "regexper library")
    )
Maintainer: Qin Wenfeng <mail@qinwenfeng.com>
Description: RE2 <https://github.com/google/re2> is a primarily deterministic finite automaton based regular expression engine from Google that is very fast
    at matching large amounts of text. 
License: BSD_3_clause + file LICENSE
LazyData: TRUE
Depends: R (>= 3.3)
Imports: Rcpp (>= 0.12.2), stringi, RcppParallel, htmlwidgets (>= 0.6),
LinkingTo: Rcpp, RcppParallel
Suggests: knitr (>= 1.12.3),testthat, rmarkdown (>=
        0.9.5),microbenchmark, ggplot2, directlabels
URL: https://github.com/qinwf/re2r/
BugReports: https://github.com/qinwf/re2r/issues
VignetteBuilder: knitr
NeedsCompilation: yes
RoxygenNote: 6.0.1
SystemRequirements: GNU make
Packaged: 2017-09-04 15:31:28 UTC; qinwf
Author: Qin Wenfeng [aut, cre],
  Toby Dylan Hocking [ctb] (benchmarks),
  Marek Gagolewski [ctb] (stringi benchmarks and test cases),
  RE2 developers [ctb] (RE2 library),
  Google Inc. [ctb, cph] (RE2 library),
  Lucent Technologies [ctb, cph] (RE2 library),
  Rob Pike [ctb] (RE2 library),
  Ken Thompson [ctb] (RE2 library),
  Julian Seward [ctb, cph] (valgrind.h),
  Andrzej Krzemienski [ctb] (optional.hpp),
  Jeffrey Avallone [ctb] (regexper library)
Repository: CRAN
Date/Publication: 2017-09-04 17:32:09 UTC
Built: R 4.4.1; x86_64-apple-darwin21.6.0; 2024-06-16 17:03:11 UTC; unix
