# Generated by roxygen2: do not edit by hand

S3method("$",re2_matrix)
S3method(print,re2_matrix)
S3method(print,re2c)
export("sub_string<-")
export(ANCHOR_BOTH)
export(ANCHOR_START)
export(UNANCHORED)
export(get_expression_size)
export(get_named_groups)
export(get_number_of_groups)
export(get_options)
export(get_pattern)
export(get_program_fanout)
export(get_simplify)
export(is_re2c_na)
export(quote_meta)
export(re2)
export(re2_count)
export(re2_detect)
export(re2_extract)
export(re2_extract_all)
export(re2_locate)
export(re2_locate_all)
export(re2_match)
export(re2_match_all)
export(re2_replace)
export(re2_replace_all)
export(re2_split)
export(re2_split_fixed)
export(re2_subset)
export(show_regex)
export(sub_string)
import(Rcpp)
import(htmlwidgets)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(stringi,stri_enc_toutf8)
useDynLib(re2r)
