sd_section("Match",
  "Regular Expression Related Functions",
  c(
    "re2_detect",
    "re2_extract",
    "re2_locate",
    "re2_match",
    "re2_replace",
    "re2_split",
    "re2_subset"
  )
)

sd_section("Compile RE2 Regular Expression",
  "RE2 Object",
  c(
    "re2",
    "get_expression_size",
    "get_named_groups",
    "get_number_of_groups",
    "get_pattern",
    "get_program_fanout",
    "print.re2c"
  )
)


sd_section("Utilities",
  "Helper Functions",
  c(
    "show_regex",
    "quote_meta",
    "sub_string"
  )
)
